@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -36,7 +36,7 @@ ConfDialog::ConfDialog( CHtmlSysWinGroupQt* parent )
     Settings* sett = qFrame->settings();
     sett->loadFromDisk();
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     // On the Mac, make the color selection buttons smaller so that they
     // become square instead of round.
     QSize macSize(48, 24);
@@ -135,11 +135,11 @@ ConfDialog::ConfDialog( CHtmlSysWinGroupQt* parent )
     }
     qSort(codecs);
     for (int i = 0; i < codecs.size(); ++i) {
-        if (ui->encodingComboBox->findText(QString::fromAscii(codecs.at(i))) == -1) {
-            ui->encodingComboBox->addItem(QString::fromAscii(codecs.at(i)));
+        if (ui->encodingComboBox->findText(QString::fromLatin1(codecs.at(i))) == -1) {
+            ui->encodingComboBox->addItem(QString::fromLatin1(codecs.at(i)));
         }
     }
-    ui->encodingComboBox->setCurrentIndex(ui->encodingComboBox->findText(QString::fromAscii(sett->tads2Encoding)));
+    ui->encodingComboBox->setCurrentIndex(ui->encodingComboBox->findText(QString::fromLatin1(sett->tads2Encoding)));
 
     ui->softScrollCheckBox->setChecked(sett->softScrolling);
     ui->askForGameFileCheckBox->setChecked(sett->askForGameFile);
@@ -153,7 +153,7 @@ ConfDialog::ConfDialog( CHtmlSysWinGroupQt* parent )
       default: ui->updateNeverRadioButton->setChecked(true); break;
     }
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     // On Mac OS X, the dialog should not have any buttons, and settings
     // changes should apply instantly.
     this->fMakeInstantApply();
@@ -321,7 +321,7 @@ ConfDialog::fApplySettings()
     } else if (ui->safetyWrite4RadioButton->isChecked()) {
         sett->ioSafetyLevelWrite = 4;
     }
-    sett->tads2Encoding = ui->encodingComboBox->currentText().toAscii();
+    sett->tads2Encoding = ui->encodingComboBox->currentText().toLatin1();
     sett->softScrolling = ui->softScrollCheckBox->isChecked();
     sett->askForGameFile = ui->askForGameFileCheckBox->isChecked();
     sett->confirmRestartGame = ui->confirmRestartCheckBox->isChecked();
