@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -19,6 +19,8 @@
 
 #include <QDialog>
 
+#include "config.h"
+
 
 namespace Ui {
     class ConfDialog;
@@ -29,11 +31,11 @@ class ConfDialog: public QDialog {
 
   public:
     ConfDialog( class CHtmlSysWinGroupQt* parent = 0 );
-    ~ConfDialog();
+    ~ConfDialog() override;
 
   protected:
     void
-    changeEvent( QEvent* e );
+    changeEvent( QEvent* e ) override;
 
   private:
     Ui::ConfDialog* ui;
