@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -17,7 +17,7 @@
 #include <QMetaType>
 #include <QFileDialog>
 
-#ifndef Q_WS_ANDROID
+#ifndef Q_OS_ANDROID
 #include <SDL.h>
 #include <SDL_mixer.h>
 #include <SDL_sound.h>
@@ -41,7 +41,7 @@
 
 // Static OS X builds need the Qt codec plugins.
 #ifndef NO_STATIC_TEXTCODEC_PLUGINS
-#  if defined(Q_WS_MAC)
+#  if defined(Q_OS_MAC)
 #    include <QtPlugin>
      Q_IMPORT_PLUGIN(qcncodecs)
      Q_IMPORT_PLUGIN(qjpcodecs)
@@ -64,10 +64,10 @@ int main( int argc, char** argv )
     if (args.size() == 2) {
         if (QFile::exists(args.at(1))) {
             gameFileName = args.at(1);
-        } else if (QFile::exists(args.at(1) + QString::fromAscii(".gam"))) {
-            gameFileName = args.at(1) + QString::fromAscii(".gam");
-        } else if (QFile::exists(args.at(1) + QString::fromAscii(".t3"))) {
-            gameFileName = args.at(1) + QString::fromAscii(".t3");
+        } else if (QFile::exists(args.at(1) + QString::fromLatin1(".gam"))) {
+            gameFileName = args.at(1) + QString::fromLatin1(".gam");
+        } else if (QFile::exists(args.at(1) + QString::fromLatin1(".t3"))) {
+            gameFileName = args.at(1) + QString::fromLatin1(".t3");
         } else {
             qWarning() << "File" << args.at(1) << "not found.";
         }
@@ -75,12 +75,12 @@ int main( int argc, char** argv )
 
     if (gameFileName.isEmpty() and app->settings()->askForGameFile) {
         gameFileName = QFileDialog::getOpenFileName(0, QObject::tr("Choose the TADS game you wish to run"),
-                                                    QString::fromAscii(""),
+                                                    QString::fromLatin1(""),
                                                     QObject::tr("TADS Games")
-                                                    + QString::fromAscii("(*.gam *.Gam *.GAM *.t3 *.T3)"));
+                                                    + QString::fromLatin1("(*.gam *.Gam *.GAM *.t3 *.T3)"));
     }
 
-#ifndef Q_WS_ANDROID
+#ifndef Q_OS_ANDROID
     if (SDL_Init(SDL_INIT_AUDIO) != 0) {
         qWarning("Unable to initialize sound system: %s", SDL_GetError());
         return 1;
@@ -147,7 +147,7 @@ int main( int argc, char** argv )
     int ret = app->exec();
 
     delete app;
-#ifndef Q_WS_ANDROID
+#ifndef Q_OS_ANDROID
     Mix_ChannelFinished(0);
     Mix_HookMusicFinished(0);
     // Close the audio device as many times as it was opened.
