@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -25,7 +25,11 @@
  */
 
 #include <QApplication>
-#include <QDesktopServices>
+#if QT_VERSION < 0x050000
+    #include <QDesktopServices>
+#else
+    #include <QStandardPaths>
+#endif
 #include <QDir>
 #include <QDateTime>
 #include <QTimer>
@@ -49,47 +53,118 @@
 #include "syswininput.h"
 
 
-/* --------------------------------------------------------------------
- * Basic file I/O interface.
- *
- * There's no need to implement this in a Qt-specific way, since we use only
- * portable functions from the standard C-library.  Qt is only used when it
- * makes things simpler or no Standard C alternative exists.
- *
- * Note that the code doesn't care if the system distinguishes between text and
- * binary files, since (as far as I know) the standard functions always do the
- * right thing; a "b" in the mode string is ignored on systems that treat text
- * and binary files the same (like most/all POSIX-systems).
+/* Translates a local filename to a unicode QString.
  */
-
-/* Open text file for reading and writing.
- */
-osfildef*
-osfoprwt( const char* fname, os_filetype_t /*filetype*/ )
+QString
+fnameToQStr( const char* fname )
 {
+    Q_ASSERT(qFrame != 0);
     Q_ASSERT(fname != 0);
 
-    // Try opening the file in read/write mode.
-    osfildef* fp = std::fopen(fname, "r+");
-
-    // If opening the file failed, it probably means that it doesn't exist.  In
-    // that case, create a new file in read/write mode.
-    if (fp == 0) fp = std::fopen(fname, "w+");
-    return fp;
+    if (qFrame->tads3())
+        return QString::fromUtf8(fname);
+    return QFile::decodeName(fname);
 }
 
 
-/* Open binary file for reading/writing.
+/* Translates a unicode QString filename to a local filename.
  */
-osfildef*
-osfoprwb( const char* fname, os_filetype_t filetype )
+QByteArray
+qStrToFname( const QString& fnameStr )
+{
+    if (qFrame->tads3())
+        return fnameStr.toUtf8();
+    return QFile::encodeName(fnameStr);
+}
+
+
+/* --------------------------------------------------------------------
+ * Basic file I/O interface.
+ */
+
+static osfildef*
+createQFile( const char* fname, QFile::OpenMode mode )
 {
     Q_ASSERT(fname != 0);
-    Q_ASSERT(filetype != OSFTLOG);
 
-    osfildef* fp = std::fopen(fname, "r+b");
-    if (fp == 0) fp = std::fopen(fname, "w+b");
-    return fp;
+    QFile* file = new QFile(fnameToQStr(fname));
+    if (not file->open(mode)) {
+        delete file;
+        return 0;
+    }
+    return file;
+}
+
+
+/* Open text file for reading.
+ */
+osfildef*
+osfoprt( const char* fname, os_filetype_t )
+{
+    return createQFile(fname, QFile::ReadOnly | QFile::Text);
+}
+
+
+/* Open text file for writing.
+ */
+osfildef*
+osfopwt( const char* fname, os_filetype_t )
+{
+    return createQFile(fname, QFile::WriteOnly | QFile::Truncate | QFile::Text);
+}
+
+
+/* Open text file for reading and writing, keeping existing contents.
+ */
+osfildef*
+osfoprwt( const char* fname, os_filetype_t )
+{
+    return createQFile(fname, QFile::ReadWrite | QFile::Text);
+}
+
+
+/* Open text file for reading and writing, truncating existing contents.
+ */
+osfildef*
+osfoprwtt( const char* fname, os_filetype_t )
+{
+    return createQFile(fname, QFile::ReadWrite | QFile::Truncate | QFile::Text);
+}
+
+
+/* Open binary file for writing.
+ */
+osfildef*
+osfopwb( const char* fname, os_filetype_t )
+{
+    return createQFile(fname, QFile::WriteOnly | QFile::Truncate);
+}
+
+
+/* Open binary file for reading.
+ */
+osfildef*
+osfoprb( const char* fname, os_filetype_t )
+{
+    return createQFile(fname, QFile::ReadOnly);
+}
+
+
+/* Open binary file for reading and writing, keeping any existing contents.
+ */
+osfildef*
+osfoprwb( const char* fname, os_filetype_t )
+{
+    return createQFile(fname, QFile::ReadWrite);
+}
+
+
+/* Open binary file for reading and writing, truncating existing contents.
+ */
+osfildef*
+osfoprwtb( const char* fname, os_filetype_t )
+{
+    return createQFile(fname, QFile::ReadWrite | QFile::Truncate);
 }
 
 
@@ -98,50 +173,68 @@ osfoprwb( const char* fname, os_filetype_t filetype )
 osfildef*
 osfdup( osfildef* orig, const char* mode )
 {
-    char realmode[5];
-    char *p = realmode;
-    const char *m;
+    Q_ASSERT(orig != 0);
+    Q_ASSERT(mode != 0);
+    Q_ASSERT(mode[0] != '\0');
 
-    /* verify that there aren't any unrecognized mode flags */
-    for (m = mode ; *m != '\0' ; ++m)
-    {
-        if (strchr("rw+bst", *m) == 0)
-            return 0;
+    QFile::OpenMode qMode = 0;
+    size_t i = 0;
+
+    if (mode[i] == 'r') {
+        ++i;
+        if (mode[i] == '+') {
+            qMode = QFile::ReadWrite;
+            ++i;
+        } else {
+            qMode = QFile::ReadOnly;
+        }
+    } else if (mode[0] == 'w') {
+        ++i;
+        qMode = QFile::WriteOnly;
+    } else {
+        return 0;
     }
 
-    /* figure the read/write mode - translate r+ and w+ to r+ */
-    if ((mode[0] == 'r' || mode[0] == 'w') && mode[1] == '+')
-        *p++ = 'r', *p++ = '+';
-    else if (mode[0] == 'r')
-        *p++ = 'r';
-    else if (mode[0] == 'w')
-        *p++ = 'w';
-    else
+    if (mode[i] == 't' or mode[i] == 's' or mode[i] == '\0') {
+        qMode |= QFile::Text;
+    } else if (mode[i] == 'b') {
+        // Binary mode is the default; there's no explicit flag for it.
+    } else {
         return 0;
+    }
 
-    /* end the mode string */
-    *p = '\0';
-
-    /* duplicate the handle in the given mode */
-#ifdef Q_OS_WIN32
-    return _fdopen(_dup(_fileno(orig)), mode);
-#else
-    return fdopen(dup(fileno(orig)), mode);
-#endif
+    QFile* file = new QFile(0);
+    if (not file->open(orig->handle(), qMode)) {
+        delete file;
+        return 0;
+    }
+    return file;
 }
 
 
-#if 0
-/* Print a counted-length string (which might not be null-terminated)
- * to a file.
+/* Get a line of text from a text file.
  */
-void
-os_fprint( osfildef* fp, const char* str, size_t len )
+char*
+osfgets( char* buf, size_t len, osfildef* fp )
 {
+    Q_ASSERT(buf != 0);
     Q_ASSERT(fp != 0);
-    Q_ASSERT(str != 0);
 
-    std::fprintf(fp, "%.*s", static_cast<unsigned>(len), str);
+    if (fp->readLine(buf, len) != static_cast<qint64>(len)) {
+        return 0;
+    }
+    return buf;
+}
+
+
+/* Write a line of text to a text file.
+ */
+int
+osfputs( const char* buf, osfildef* fp )
+{
+    if (fp->write(buf, qstrlen(buf)) < 1)
+        return EOF;
+    return 1;
 }
 
 
@@ -153,23 +246,150 @@ os_fprintz( osfildef* fp, const char* str )
     Q_ASSERT(fp != 0);
     Q_ASSERT(str != 0);
 
-    std::fprintf(fp, "%s", str);
+    fp->write(str, qstrlen(str));
 }
-#endif
 
 
+/* Print a counted-length string (which might not be null-terminated)
+ * to a file.
+ */
+void
+os_fprint( osfildef* fp, const char* str, size_t len )
+{
+    Q_ASSERT(fp != 0);
+    Q_ASSERT(str != 0);
+
+    fp->write(str, len);
+}
+
+
+/* Write bytes to file.
+ */
+int
+osfwb( osfildef* fp, const void* buf, int bufl )
+{
+    Q_ASSERT(fp != 0);
+    Q_ASSERT(buf != 0);
+
+    if (fp->write(static_cast<const char*>(buf), bufl) < 1)
+        return 1;
+    return 0;
+}
+
+
+/* Flush buffered writes to a file.
+ */
+int
+osfflush( osfildef* fp )
+{
+    Q_ASSERT(fp != 0);
+
+    if (not fp->flush())
+        return EOF;
+    return 0;
+}
+
+
+/* Get a character from a file.
+ */
+int
+osfgetc( osfildef* fp )
+{
+    Q_ASSERT(fp != 0);
+
+    char c;
+    if (not fp->getChar(&c))
+        return EOF;
+    return c;
+}
+
+
+/* Read bytes from file.
+ */
+int
+osfrb( osfildef* fp, void* buf, int bufl )
+{
+    Q_ASSERT(fp != 0);
+    Q_ASSERT(buf != 0);
+
+    return fp->read(static_cast<char*>(buf), bufl) != bufl;
+}
+
+
+/* Read bytes from file and return the number of bytes read.
+ */
+size_t
+osfrbc( osfildef* fp, void* buf, size_t bufl )
+{
+    Q_ASSERT(fp != 0);
+    Q_ASSERT(buf != 0);
+
+    int bytesRead = fp->read(static_cast<char*>(buf), bufl);
+    if (bytesRead < 0)
+        return 0;
+    return bytesRead;
+}
+
+
+/* Get the current seek location in the file.
+ */
+long
+osfpos( osfildef* fp )
+{
+    Q_ASSERT(fp != 0);
+
+    return fp->pos();
+}
+
+
+/* Seek to a location in the file.
+ */
+int
+osfseek( osfildef* fp, long pos, int mode )
+{
+    Q_ASSERT(fp != 0);
+
+    if (mode == OSFSK_CUR)
+        pos += fp->pos();
+    else if (mode == OSFSK_END)
+        pos += fp->size();
+    return not fp->seek(pos);
+}
+
+
+/* Close a file.
+ */
+void
+osfcls( osfildef* fp )
+{
+    delete fp;
+}
+
+
+/* Delete a file.
+ */
+int
+osfdel( const char* fname )
+{
+    return not QFile::remove(fnameToQStr(fname));
+}
+
+
+/* Rename a file.
+ */
 int
 os_rename_file( const char* oldname, const char* newname )
 {
-    return QFile::rename(QFile::decodeName(oldname),
-                         QFile::decodeName(newname));
+    return QFile::rename(fnameToQStr(oldname), fnameToQStr(newname));
 }
 
 
+/* Access a file - determine if the file exists.
+ */
 int
 osfacc( const char* fname )
 {
-    const QFileInfo info(QFile::decodeName(fname));
+    const QFileInfo info(fnameToQStr(fname));
     // Since exists() returns false for dangling symlinks, we need
     // to explicitly check whether it's a symlink or not.
     if (not info.exists() and not info.isSymLink()) {
@@ -187,13 +407,22 @@ extern Q_CORE_EXPORT int qt_ntfs_permission_lookup;
 int
 os_file_stat( const char* fname, int follow_links, os_file_stat_t* s )
 {
+    Q_ASSERT(fname != 0);
+    Q_ASSERT(s != 0);
+
+    const QString& fnameStr = fnameToQStr(fname);
+
 #ifdef Q_OS_WIN32
     struct __stat64 info;
 
-    // $$$ we should support Windows symlinks and junction points
+    // Get a UTF-16 version of the filename and NULL-terminate it.
+    wchar_t* wFname = new wchar_t[fnameStr.length() + 1];
+    wFname[fnameStr.toWCharArray(wFname)] = L'\0';
 
-    /* get the file information */
-    if (_stat64(fname, &info))
+    // Get the file information.
+    int statRet = _wstat64(wFname, &info);
+    delete[] wFname;
+    if (statRet != 0)
         return FALSE;
 
     /* translate the status fields */
@@ -205,7 +434,12 @@ os_file_stat( const char* fname, int follow_links, os_file_stat_t* s )
     s->mode = info.st_mode;
 #else
     struct stat buf;
-    if ((follow_links ? stat(fname, &buf) : lstat(fname, &buf)) != 0)
+    int statRet;
+    if (follow_links)
+        statRet = stat(QFile::encodeName(fnameStr).constData(), &buf);
+    else
+        statRet = lstat(QFile::encodeName(fnameStr).constData(), &buf);
+    if (statRet != 0)
         return false;
 
     s->sizelo = (uint32_t)(buf.st_size & 0xFFFFFFFF);
@@ -224,7 +458,7 @@ os_file_stat( const char* fname, int follow_links, os_file_stat_t* s )
     if (osfacc(fname) != 0) {
         return false;
     }
-    QFileInfo inf(QFile::decodeName(fname));
+    QFileInfo inf(fnameStr);
     bool isLink = inf.isSymLink();
 #ifdef Q_OS_WIN32
     // Don't treat shortcut files as symlinks.
@@ -267,8 +501,8 @@ os_file_stat( const char* fname, int follow_links, os_file_stat_t* s )
 int
 os_resolve_symlink( const char* fname, char* target, size_t target_size )
 {
-    const QByteArray& str = QFile::encodeName(QFileInfo(QFile::decodeName(fname)).symLinkTarget());
-    if (str.isEmpty() or str.size() >= target_size) {
+    const QByteArray& str = qStrToFname(QFileInfo(fnameToQStr(fname)).symLinkTarget());
+    if (str.isEmpty() or str.size() >= static_cast<int>(target_size)) {
         return false;
     }
     qstrcpy(target, str.constData());
@@ -296,7 +530,7 @@ os_get_root_dirs( char* buf, size_t buflen )
     // The whole result must end with two NULL bytes.
     str += '\0';
 
-    if (buf != 0 and buflen >= str.size()) {
+    if (buf != 0 and static_cast<int>(buflen) >= str.size()) {
         memcpy(buf, str.constData(), str.size());
     }
     return str.size();
@@ -306,7 +540,7 @@ os_get_root_dirs( char* buf, size_t buflen )
 int
 os_open_dir( const char* dirname, osdirhdl_t* handle )
 {
-    QDirIterator* d = new QDirIterator(QFile::decodeName(dirname),
+    QDirIterator* d = new QDirIterator(fnameToQStr(dirname),
                                        QDir::Dirs | QDir::Files | QDir::Hidden | QDir::System);
     if (d->next().isEmpty()) {
         // We can't read anything.  Don't know why, don't care.
@@ -320,8 +554,8 @@ os_open_dir( const char* dirname, osdirhdl_t* handle )
 int
 os_read_dir( osdirhdl_t handle, char* fname, size_t fname_size )
 {
-    const QByteArray& str = QFile::encodeName(handle->fileName());
-    if (str.isEmpty() or str.size() >= fname_size) {
+    const QByteArray& str = qStrToFname(handle->fileName());
+    if (str.isEmpty() or str.size() >= static_cast<int>(fname_size)) {
         return false;
     }
     qstrcpy(fname, str.constData());
@@ -405,41 +639,22 @@ os_exeseek( const char*, const char* )
 
 /* Get the full filename (including directory path) to the executable
  * file.
- *
- * The implementation provided here can handle links.  (Both Unix-links
- * as well as Windows-shortcuts, since Qt supports then both; I don't
- * know how a link looks like on a Mac, but Qt should support that
- * too.)
- *
- * TODO: Search through the PATH env. variable.  Find out how this is
- * supposed to work on OS X.
  */
 int
-os_get_exe_filename( char* buf, size_t buflen, const char* argv0 )
+os_get_exe_filename( char* buf, size_t buflen, const char* )
 {
-    QFileInfo file(QFile::decodeName(argv0));
-    file.makeAbsolute();
-    if (not file.exists() or not file.isReadable()) {
+    QFileInfo inf(QApplication::applicationFilePath());
+
+    if (not inf.exists() or not inf.isReadable()) {
         return false;
     }
 
-    // If the file is some form of link, find out where it points to.
-    if (file.isSymLink()) {
-        while (not file.readLink().isEmpty()) {
-            file.setFile(file.readLink());
-        }
-        file.makeAbsolute();
-        if (not file.exists() or not file.isReadable()) {
-            return false;
-        }
-    }
-
-    const QByteArray& result = QFile::encodeName(file.filePath());
-    if (result.length() + 1 > static_cast<int>(buflen)) {
+    const QByteArray& encodedFilename = qStrToFname(inf.filePath());
+    if (encodedFilename.length() + 1 > static_cast<int>(buflen)) {
         // The result would not fit in the buffer.
         return false;
     }
-    strcpy(buf, result.constData());
+    qstrcpy(buf, encodedFilename.constData());
     return true;
 }
 
@@ -460,33 +675,33 @@ os_get_special_path( char* buf, size_t buflen, const char* /*argv0*/, int id )
         // We can safely ignore those. They're needed only by the compiler.
         // OS_GSP_T3_RES is only needed by the base code implementation of
         // charmap.cc (tads3/charmap.cpp) which we don't use.
+
+        // FIXME: We do use tads3/charmap.cpp, so this needs to be handled.
         return;
 
-      case OS_GSP_T3_APP_DATA: {
-        const QString& dirStr = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+      case OS_GSP_T3_APP_DATA:
+      case OS_GSP_LOGFILE: {
+        const QString& dirStr =
+        #if QT_VERSION < 0x050000
+                QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+        #else
+                QStandardPaths::writableLocation(QStandardPaths::DataLocation);
+        #endif
         QDir dir(dirStr);
         // Create the directory if it doesn't exist.
         if (not dir.exists() and not dir.mkpath(dirStr)) {
             // TODO: Error dialog.
             qWarning() << "Could not create directory path:" << dirStr;
-            Q_ASSERT(QFile::encodeName(QDir::tempPath()).size() < static_cast<int>(buflen));
-            strncpy(buf, QFile::encodeName(QDir::tempPath()).constData(), buflen);
+            Q_ASSERT(qStrToFname(QDir::tempPath()).size() < static_cast<int>(buflen));
+            strncpy(buf, qStrToFname(QDir::tempPath()).constData(), buflen);
             return;
         }
-        Q_ASSERT(QFile::encodeName(dirStr).size() < static_cast<int>(buflen));
-        strncpy(buf, QFile::encodeName(dirStr).constData(), buflen);
+        Q_ASSERT(qStrToFname(dirStr).size() < static_cast<int>(buflen));
+        strncpy(buf, qStrToFname(dirStr).constData(), buflen);
         buf[buflen - 1] = '\0';
         break;
       }
 
-      case OS_GSP_LOGFILE: {
-        // TODO: We'll just use the temp directory for now.
-        const QByteArray& res = QFile::encodeName(QDir::tempPath());
-        Q_ASSERT(res.size() < static_cast<int>(buflen));
-        qstrncpy(buf, res.constData(), buflen);
-        break;
-      }
-
       default:
         // We didn't recognize the specified id. That means the base code
         // added a new value for it that we don't know about.
@@ -512,8 +727,8 @@ os_locate( const char* fname, int /*flen*/, const char* /*arg0*/, char* buf, siz
     Q_ASSERT(fname != 0);
     Q_ASSERT(buf != 0);
 
-    const QFileInfo& fileInfo = QFileInfo(QFile::decodeName(fname));
-    const QByteArray& result = QFile::encodeName(fileInfo.absoluteFilePath());
+    const QFileInfo& fileInfo = QFileInfo(fnameToQStr(fname));
+    const QByteArray& result = qStrToFname(fileInfo.absoluteFilePath());
     if (bufsiz > result.length() and QFile::exists(fileInfo.absoluteFilePath())) {
         strcpy(buf, result.constData());
         return true;
@@ -534,14 +749,19 @@ os_create_tempfile( const char* fname, char* buf )
 {
     if (fname != 0 and fname[0] != '\0') {
         // A filename has been specified; use it.
-        return std::fopen(fname, "w+b");
+        return createQFile(fname, QFile::ReadWrite | QFile::Truncate);
     }
 
     Q_ASSERT(buf != 0);
 
-    // No filename needed; create a nameless temp-file.
+    // No filename was given; create a temporary file.
     buf[0] = '\0';
-    return std::tmpfile();
+    QTemporaryFile* file = new QTemporaryFile(QString::fromLatin1("/qtads_XXXXXX"));
+    if (not file->open()) {
+        delete file;
+        return 0;
+    }
+    return file;
 }
 
 
@@ -552,9 +772,9 @@ osfdel_temp( const char* fname )
 {
     Q_ASSERT(fname != 0);
 
-    if (fname[0] == '\0' or QFile::remove(QFile::decodeName(fname))) {
+    if (fname[0] == '\0' or QFile::remove(fnameToQStr(fname))) {
         // If fname was empty, it has been already deleted automatically by
-        // fclose().  If fname was not empty, QFile::remove has taken care of
+        // osfcls().  If fname was not empty, QFile::remove has taken care of
         // deleting the file.
         return 0;
     }
@@ -563,20 +783,33 @@ osfdel_temp( const char* fname )
 }
 
 
+/* This isn't actually used by the basecode.
+ */
+#if 0
+void
+os_get_tmp_path( char* buf )
+{
+}
+#endif
+
+
 /* Generate a name for a temporary file.
  */
 int
 os_gen_temp_filename( char* buf, size_t buflen )
 {
-    QTemporaryFile tmpfile(QDir::tempPath() + QString::fromAscii("/qtads_XXXXXX"));
+    QTemporaryFile tmpfile(QDir::tempPath() + QString::fromLatin1("/qtads_XXXXXX"));
+    if (not tmpfile.open()) {
+        return false;
+    }
+
     // Don't automatically delete the file from disk. This is safer,
     // since another process could create a file with the same name
     // before our caller gets the chance to re-create the file.
     tmpfile.setAutoRemove(false);
-    tmpfile.open();
-    const QByteArray& data = QFile::encodeName(tmpfile.fileName());
-    tmpfile.close();
-    if (data.length() >= buflen) {
+
+    const QByteArray& data = qStrToFname(tmpfile.fileName());
+    if (data.length() >= static_cast<int>(buflen)) {
         // 'buf' isn't big enough to hold the result, including the
         // terminating '\0'.
         return false;
@@ -596,9 +829,9 @@ int
 os_mkdir( const char* dir, int create_parents )
 {
     if (create_parents) {
-        return QDir().mkpath(QFile::decodeName(dir));
+        return QDir().mkpath(fnameToQStr(dir));
     }
-    return QDir().mkdir(QFile::decodeName(dir));
+    return QDir().mkdir(fnameToQStr(dir));
 }
 
 
@@ -607,7 +840,7 @@ os_mkdir( const char* dir, int create_parents )
 int
 os_rmdir( const char* dir )
 {
-    return QDir().rmdir(QFile::decodeName(dir));
+    return QDir().rmdir(fnameToQStr(dir));
 }
 
 
@@ -615,7 +848,6 @@ os_rmdir( const char* dir )
  * Filename manipulation routines.
  */
 
-#if 0
 /* Apply a default extension to a filename, if it doesn't already have one.
  */
 void
@@ -624,7 +856,7 @@ os_defext( char* fn, const char* ext )
     Q_ASSERT(fn != 0);
     Q_ASSERT(ext != 0);
 
-    if (QFileInfo(QFile::decodeName(fn)).suffix().isEmpty()) {
+    if (QFileInfo(fnameToQStr(fn)).suffix().isEmpty()) {
         os_addext(fn, ext);
     }
 }
@@ -653,11 +885,11 @@ os_remext( char* fn )
 {
     Q_ASSERT(fn != 0);
 
-    QFileInfo file(QFile::decodeName(fn));
+    QFileInfo file(fnameToQStr(fn));
     if (file.suffix().isEmpty()) {
         return;
     }
-    std::strcpy(fn, QFile::encodeName(file.completeBaseName()));
+    std::strcpy(fn, qStrToFname(file.completeBaseName()));
 }
 
 
@@ -665,6 +897,8 @@ os_remext( char* fn )
  *
  * Note that Qt's native path separator character is '/'.  It doesn't matter on
  * what OS we're running.
+ *
+ * FIXME: This only works for UTF-8 (T3 only).
  */
 char*
 os_get_root_name( const char* buf )
@@ -690,11 +924,35 @@ os_build_full_path( char* fullpathbuf, size_t fullpathbuflen, const char* path,
     Q_ASSERT(path != 0);
     Q_ASSERT(filename != 0);
 
-    std::strncpy(fullpathbuf,
-                 QFile::encodeName(QFileInfo(QDir(QFile::decodeName(path)),
-                                             QFile::decodeName(filename)).filePath()),
-                 fullpathbuflen);
-    fullpathbuf[fullpathbuflen - 1] = '\0';
+    qstrncpy(fullpathbuf, qStrToFname(QFileInfo(QDir(fnameToQStr(path)),
+                                                fnameToQStr(filename)).filePath()),
+             fullpathbuflen);
+}
+
+
+void
+os_combine_paths( char* fullpathbuf, size_t fullpathbuflen, const char* path, const char* filename )
+{
+    Q_ASSERT(fullpathbuf != 0);
+    Q_ASSERT(path != 0);
+    Q_ASSERT(filename != 0);
+
+    bool filenameIsDot = false;
+    bool filenameIsDotDot = false;
+    if (qstrcmp(filename, "..") == 0 or qstrcmp(filename, "../") == 0)
+        filenameIsDotDot = true;
+    else if (qstrcmp(filename, ".") == 0 or qstrcmp(filename, "./") == 0) {
+        filenameIsDot = true;
+    }
+
+    if (filenameIsDot or filenameIsDotDot) {
+        os_build_full_path(fullpathbuf, fullpathbuflen, path, "");
+        if (qstrlen(fullpathbuf) + 3 <= fullpathbuflen) {
+            strcat(fullpathbuf, filenameIsDot ? "." : "..");
+        }
+    } else {
+        os_build_full_path(fullpathbuf, fullpathbuflen, path, filename);
+    }
 }
 
 
@@ -703,22 +961,24 @@ os_build_full_path( char* fullpathbuf, size_t fullpathbuflen, const char* path,
 void
 os_get_path_name( char* pathbuf, size_t pathbuflen, const char* fname )
 {
-    strncpy(pathbuf, QFile::encodeName(QFileInfo(QFile::decodeName(fname)).path()).constData(), pathbuflen);
-    pathbuf[pathbuflen - 1] = '\0';
+    Q_ASSERT(pathbuf != 0);
+    Q_ASSERT(fname != 0);
+
+    qstrncpy(pathbuf, qStrToFname(QFileInfo(fnameToQStr(fname)).path()).constData(),
+             pathbuflen);
 }
 
 
 /* Convert a relative URL into a relative filename path.
  */
 void
-os_cvt_url_dir( char* result_buf, size_t result_buf_size, const char* src_url, int end_sep )
+os_cvt_url_dir( char* result_buf, size_t result_buf_size, const char* src_url )
 {
-    QString result(QFile::decodeName(src_url));
-    if (end_sep == true and not result.endsWith(QChar::fromAscii('/'))) {
-        result.append(QChar::fromAscii('/'));
-    }
-    strncpy(result_buf, QFile::encodeName(result).constData(), result_buf_size);
-    result_buf[result_buf_size - 1] = '\0';
+    Q_ASSERT(result_buf != 0);
+    Q_ASSERT(src_url != 0);
+
+    QString result(fnameToQStr(src_url));
+    qstrncpy(result_buf, qStrToFname(result).constData(), result_buf_size);
 }
 
 
@@ -727,9 +987,10 @@ os_cvt_url_dir( char* result_buf, size_t result_buf_size, const char* src_url, i
 int
 os_is_file_absolute( const char* fname )
 {
-    return QFileInfo(QFile::decodeName(fname)).isAbsolute();
+    Q_ASSERT(fname != 0);
+
+    return QFileInfo(fnameToQStr(fname)).isAbsolute();
 }
-#endif
 
 
 /* Get the absolute, fully qualified filename for a file.
@@ -738,17 +999,39 @@ int
 os_get_abs_filename( char* result_buf, size_t result_buf_size, const char* filename )
 {
     Q_ASSERT(result_buf != 0);
-    const QByteArray& data = QFile::encodeName(QFileInfo(QFile::decodeName(filename)).absoluteFilePath());
-    if (data.length() >= result_buf_size) {
-        // Result won't fit in 'result_buf'.
-        qstrcpy(result_buf, filename);
+
+    const QByteArray& data = qStrToFname(QFileInfo(fnameToQStr(filename)).absoluteFilePath());
+    qstrncpy(result_buf, data.constData(), result_buf_size);
+    if (data.length() >= static_cast<int>(result_buf_size)) {
+        // Result didn't fit in 'result_buf'.
         return false;
     }
-    qstrcpy(result_buf, data.constData());
     return true;
 }
 
 
+// This is only used by the HTML TADS debugger, which we don't include
+// in our build.
+#if 0
+int
+os_get_rel_path( char* result_buf, size_t result_buf_size, const char* basepath, const char* filename)
+{
+    Q_ASSERT(result_buf != 0);
+    Q_ASSERT(basepath != 0);
+    Q_ASSERT(filename != 0);
+
+    QDir baseDir(fnameToQStr(basepath));
+    const QString& result = baseDir.relativeFilePath(fnameToQStr(filename));
+    if (result.isEmpty()) {
+        qstrncpy(result_buf, filename, result_buf_size);
+        return false;
+    }
+    qstrncpy(result_buf, qStrToFname(result), result_buf_size);
+    return true;
+}
+#endif
+
+
 /* Determine if the given file is in the given directory.
  */
 #if 1
@@ -760,7 +1043,7 @@ canonicalize_path( char* path )
 {
     // We canonicalize only the path, in case the file doesn't actually exist.
     // QFileInfo::canonicalFilePath() doesn't work for non-existent files.
-    QFileInfo info(QFile::decodeName(path));
+    QFileInfo info(fnameToQStr(path));
     QString cleanPath;
     if (info.isDir()) {
         cleanPath = info.filePath();
@@ -768,19 +1051,20 @@ canonicalize_path( char* path )
         cleanPath = info.path();
     }
 
-    QByteArray canonPath(QFile::encodeName(QDir(cleanPath).canonicalPath()));
+    QByteArray canonPath(qStrToFname(QDir(cleanPath).canonicalPath()));
     // Append the filename if we previously stripped it.
     if (not info.isDir()) {
-        QString cleanFilename(QFile::decodeName(path));
+        QString cleanFilename(fnameToQStr(path));
         int i = cleanFilename.length();
-        while (cleanFilename[i] != QChar::fromAscii('/') and i > 0) {
+        while (cleanFilename[i] != QChar::fromLatin1('/') and i > 0) {
             --i;
         }
-        canonPath.append(QFile::encodeName(cleanFilename.mid(i)));
+        canonPath.append(qStrToFname(cleanFilename.mid(i)));
     }
     qstrncpy(path, canonPath.constData(), OSFNMAX);
 }
 
+
 int
 os_is_file_in_dir( const char* filename, const char* path, int include_subdirs,
                    int match_self )
@@ -894,8 +1178,8 @@ os_is_file_in_dir( const char* filename, const char* path, int include_subdirs )
     Q_ASSERT(filename[0] != '\0');
     Q_ASSERT(filename[qstrlen(filename) - 1] != '/');
 
-    QFileInfo fileInf(QFile::decodeName(filename));
-    const QString& pathStr = QFileInfo(QFile::decodeName(path)).canonicalFilePath();
+    QFileInfo fileInf(fnameToQStr(filename));
+    const QString& pathStr = QFileInfo(fnameToQStr(path)).canonicalFilePath();
 
     // If the filename is absolute but the file doesn't exist, we know
     // that we're not going to find it anywhere, so report failure.
@@ -945,34 +1229,45 @@ os_rand( long* val )
 {
     Q_ASSERT(val != 0);
 
-    // Is this the first call to os_rand()?
-    static bool initial = true;
-
-    if (initial) {
-        // It's the first time we're called.  Initialize the random number
-        // generator.
-        initial = false;
-        time_t t = time(0);
-        if (t == static_cast<time_t>(-1)) {
-            std::srand(std::rand());
-        } else {
-            std::srand(static_cast<unsigned int>(t));
-        }
+    static bool initialized = false;
+    if (not initialized) {
+        qsrand(QDateTime::currentDateTime().toTime_t());
+        initialized = true;
     }
-
-    // Generate a random number by using high-order bits, because on some
-    // systems the low-order bits aren't very random.
-    *val = 1 + static_cast<long>(static_cast<long double>(65535) * std::rand() / (RAND_MAX + 1.0));
+    *val = qrand();
 }
 
 
 /* Generate random bytes for use in seeding a PRNG (pseudo-random number
  * generator).
  */
-// FIXME: Implement this.
 void
-os_gen_rand_bytes( unsigned char *buf, size_t len )
+os_gen_rand_bytes( unsigned char* buf, size_t len )
 {
+#ifdef Q_OS_UNIX
+    QFile rdev(QString::fromLatin1("/dev/urandom"));
+    if (rdev.open(QFile::ReadOnly)) {
+        qint64 bytesRead = rdev.read(reinterpret_cast<char*>(buf), len);
+        if (bytesRead >= static_cast<qint64>(len))
+            return;
+        if (bytesRead > 0) {
+            // For whatever reason, we couldn't read the requested amount of
+            // bytes from the device. We'll fill the rest with the generic
+            // generator below.
+            buf += bytesRead;
+            len -= bytesRead;
+        }
+    }
+#endif
+
+    static bool initialized = false;
+    if (not initialized) {
+        qsrand(QDateTime::currentDateTime().toTime_t());
+        initialized = true;
+    }
+    for (size_t i = 0; i < len; ++i) {
+        buf[i] = qrand() % 256;
+    }
 }
 
 
@@ -1031,29 +1326,29 @@ os_askfile( const char* prompt, char* fname_buf, int fname_buf_len, int prompt_t
 
     switch (file_type) {
       case OSFTGAME:
-        filter = QObject::tr("TADS 2 Games") + QString::fromAscii(" (*.gam *.Gam *.GAM)");
+        filter = QObject::tr("TADS 2 Games") + QString::fromLatin1(" (*.gam *.Gam *.GAM)");
         break;
       case OSFTSAVE:
-        filter = QObject::tr("TADS 2 Saved Games") + QString::fromAscii(" (*.sav *.Sav *.SAV)");
+        filter = QObject::tr("TADS 2 Saved Games") + QString::fromLatin1(" (*.sav *.Sav *.SAV)");
         break;
       case OSFTLOG:
-        filter = QObject::tr("Game Transcripts") + QString::fromAscii(" (*.txt *.Txt *.TXT)");
+        filter = QObject::tr("Game Transcripts") + QString::fromLatin1(" (*.txt *.Txt *.TXT)");
         break;
       case OSFTT3IMG:
         Q_ASSERT(qFrame->tads3());
-        filter = QObject::tr("TADS 3 Games") + QString::fromAscii(" (*.t3 *.T3)");
+        filter = QObject::tr("TADS 3 Games") + QString::fromLatin1(" (*.t3 *.T3)");
         break;
       case OSFTT3SAV:
         Q_ASSERT(qFrame->tads3());
-        filter = QObject::tr("TADS 3 Saved Games") + QString::fromAscii(" (*.t3v *.T3v *.T3V)");
-        ext = QString::fromAscii("t3v");
+        filter = QObject::tr("TADS 3 Saved Games") + QString::fromLatin1(" (*.t3v *.T3v *.T3V)");
+        ext = QString::fromLatin1("t3v");
         break;
     }
 
     // Always provide an "All Files" filter.
     if (not filter.isEmpty()) {
-        filter.append(QString::fromAscii(";;"));
-        filter.append(QObject::tr("All Files") + QString::fromAscii(" (*)"));
+        filter.append(QString::fromLatin1(";;"));
+        filter.append(QObject::tr("All Files") + QString::fromLatin1(" (*)"));
     }
 
     QString promptStr;
@@ -1078,7 +1373,7 @@ os_askfile( const char* prompt, char* fname_buf, int fname_buf_len, int prompt_t
         return OS_AFE_CANCEL;
     }
 
-    const QByteArray& result = QFile::encodeName(filename);
+    const QByteArray& result = qStrToFname(filename);
     if (fname_buf_len <= result.length()) {
         return OS_AFE_FAILURE;
     }
@@ -1086,7 +1381,7 @@ os_askfile( const char* prompt, char* fname_buf, int fname_buf_len, int prompt_t
     if (not ext.isEmpty()) {
         // Since `ext' is not empty, an extension should be
         // appended (if none exists already).
-        os_defext(fname_buf, QFile::encodeName(ext).constData());
+        os_defext(fname_buf, qStrToFname(ext).constData());
         fname_buf[fname_buf_len - 1] = '\0';
     }
     return OS_AFE_SUCCESS;
@@ -1171,7 +1466,7 @@ os_input_dialog( int icon_id, const char* prompt, int standard_button_set, const
     }
     // We append a space to the window title to avoid the "<2>" that would
     // otherwise be appended automatically by some window managers (like KDE.)
-    dialog.setWindowTitle(qWinGroup->windowTitle() + QChar::fromAscii(' '));
+    dialog.setWindowTitle(qWinGroup->windowTitle() + QChar::fromLatin1(' '));
     dialog.exec();
     QAbstractButton* result = dialog.clickedButton();
     if (result == 0) {
@@ -1302,6 +1597,8 @@ os_set_save_ext( const char* )
 
 /* Generate the name of the character set mapping table for Unicode
  * characters to and from the given local character set.
+ *
+ * We use UTF-8 for everything, which should work on all platforms.
  */
 void
 os_get_charmap( char* mapname, int charmap_id )
@@ -1312,19 +1609,13 @@ os_get_charmap( char* mapname, int charmap_id )
 
     switch(charmap_id) {
       case OS_CHARMAP_DISPLAY:
-        // Always use UTF-8 for the display, regardless of the
-        // local charset; Qt uses Unicode for the display on
-        // every system.
-        strcpy(mapname, "utf8");
-        break;
       case OS_CHARMAP_FILENAME:
       case OS_CHARMAP_FILECONTENTS:
-        strcpy(mapname, QTextCodec::codecForLocale()->name());
+        strcpy(mapname, "utf-8");
         break;
       default:
         qWarning("os_get_charmap() got an unknown charmap id");
-        strcpy(mapname, QTextCodec::codecForLocale()->name());
-        break;
+        strcpy(mapname, "utf-8");
     }
 }
 
@@ -1337,8 +1628,8 @@ os_gen_charmap_filename( char* filename, char* internal_id, char* /*argv0*/ )
     qDebug() << Q_FUNC_INFO;
     Q_ASSERT(filename != 0);
 
-    strncpy(filename, QFile::encodeName(QString::fromAscii(internal_id)
-                              + QString::fromAscii(".tcp")).constData(), OSFNMAX);
+    strncpy(filename, qStrToFname(QString::fromLatin1(internal_id)
+                              + QString::fromLatin1(".tcp")).constData(), OSFNMAX);
     filename[OSFNMAX - 1] = '\0';
 }
 
@@ -1395,7 +1686,7 @@ os_get_sysinfo( int code, void* /*param*/, long* result )
       case SYSINFO_MPEG1:
       case SYSINFO_MPEG2:
       case SYSINFO_MPEG3:
-#ifndef Q_WS_ANDROID
+#ifndef Q_OS_ANDROID
         *result = 1;
 #else
         *result = 0;
@@ -1466,7 +1757,8 @@ os_get_sysinfo( int code, void* /*param*/, long* result )
  */
 // FIXME: Just a dummy implementation for now.
 int
-os_show_popup_menu( int default_pos, int x, int y, const char* txt, size_t txtlen, union os_event_info_t* evt )
+os_show_popup_menu( int /*default_pos*/, int /*x*/, int /*y*/, const char* /*txt*/,
+                    size_t /*txtlen*/, union os_event_info_t* /*evt*/ )
 {
     if (qFrame->gameRunning()) {
         return OSPOP_FAIL;
@@ -1479,12 +1771,12 @@ os_show_popup_menu( int default_pos, int x, int y, const char* txt, size_t txtle
  */
 // FIXME: Just a dummy implementation for now.
 void
-os_enable_cmd_event( int id, unsigned int status )
+os_enable_cmd_event( int /*id*/, unsigned int /*status*/ )
 {
 }
 
 
 void
-os_init_ui_after_load( class CVmBifTable* bif_table, class CVmMetaTable* meta_table)
+os_init_ui_after_load( class CVmBifTable* /*bif_table*/, class CVmMetaTable* /*meta_table*/)
 {
 }
