@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -21,6 +21,7 @@
 #include <QTime>
 
 #include "tadshtml.h"
+#include "config.h"
 
 
 /* Provides the common code for all three types of digitized sound (WAV,
@@ -33,7 +34,7 @@ class QTadsSound: public QObject {
     enum SoundType { WAV, OGG, MPEG };
 
   private:
-#ifndef Q_WS_ANDROID
+#ifndef Q_OS_ANDROID
     struct Mix_Chunk* fChunk;
     int fChannel;
     SoundType fType;
@@ -99,11 +100,10 @@ class QTadsSound: public QObject {
 
   public:
     QTadsSound( QObject* parent, struct Mix_Chunk* chunk, SoundType type );
-
-    virtual
-    ~QTadsSound();
+    ~QTadsSound() override;
 #endif
 
+  public:
     // The SDL_Mixer callback for when a sound finished playing.
     static void callback( int channel );
 
