@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -60,10 +60,10 @@ CHtmlSysFrameQt::CHtmlSysFrameQt( int& argc, char* argv[], const char* appName,
     //qDebug() << Q_FUNC_INFO;
     Q_ASSERT(qFrame == 0);
 
-    this->setApplicationName(QString::fromAscii(appName));
-    this->setApplicationVersion(QString::fromAscii(appVersion));
-    this->setOrganizationName(QString::fromAscii(orgName));
-    this->setOrganizationDomain(QString::fromAscii(orgDomain));
+    this->setApplicationName(QString::fromLatin1(appName));
+    this->setApplicationVersion(QString::fromLatin1(appVersion));
+    this->setOrganizationName(QString::fromLatin1(orgName));
+    this->setOrganizationDomain(QString::fromLatin1(orgDomain));
 
     // Load our persistent settings.
     this->fSettings = new Settings;
@@ -90,7 +90,7 @@ CHtmlSysFrameQt::CHtmlSysFrameQt( int& argc, char* argv[], const char* appName,
 
     // Create our main application window.
     this->fMainWin = new CHtmlSysWinGroupQt;
-    this->fMainWin->setWindowTitle(QString::fromAscii(appName));
+    this->fMainWin->setWindowTitle(QString::fromLatin1(appName));
     this->fMainWin->updateRecentGames();
 
     // Automatically quit the application when the last window has closed.
@@ -101,8 +101,8 @@ CHtmlSysFrameQt::CHtmlSysFrameQt( int& argc, char* argv[], const char* appName,
 
     // Set application window icon, unless we're on OS X where the bundle
     // icon is used.
-#ifndef Q_WS_MAC
-    this->setWindowIcon(QIcon(QString::fromAscii(":/qtads_48x48.png")));
+#ifndef Q_OS_MAC
+    this->setWindowIcon(QIcon(QString::fromLatin1(":/qtads_48x48.png")));
 #endif
 }
 
@@ -176,7 +176,7 @@ CHtmlSysFrameQt::fRunGame()
         QDir::setCurrent(finfo.absolutePath());
 
         // Run the appropriate TADS VM.
-        int vmType = vm_get_game_type(QFile::encodeName(finfo.absoluteFilePath()).constData(), 0, 0, 0, 0);
+        int vmType = vm_get_game_type(qStrToFname(finfo.absoluteFilePath()).constData(), 0, 0, 0, 0);
         if (vmType == VM_GGT_TADS2 or vmType == VM_GGT_TADS3) {
             // Delete all HTML and orphaned banners.
             while (not this->fOrhpanBannerList.isEmpty()) {
@@ -221,15 +221,15 @@ CHtmlSysFrameQt::fRunGame()
             // the game we're running or the game's name as at appears in the
             // gameinfo. The game is free to change that later on.
             const QString& titleStr
-                    = GameInfoDialog::getMetaInfo(QFile::encodeName(finfo.absoluteFilePath())).plainGameName;
+                    = GameInfoDialog::getMetaInfo(qStrToFname(finfo.absoluteFilePath())).plainGameName;
             if (titleStr.simplified().isEmpty()) {
                 // The game doesn't provide a game name.  Just use the filename.
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
                 // Just use the filename on OS X.  Seems to be the norm there.
                 qWinGroup->setWindowTitle(finfo.fileName());
 #else
                 // On all other systems, also append the application name.
-                qWinGroup->setWindowTitle(finfo.fileName() + QString::fromAscii(" - ") + qFrame->applicationName());
+                qWinGroup->setWindowTitle(finfo.fileName() + QString::fromLatin1(" - ") + qFrame->applicationName());
 #endif
             } else {
                 qWinGroup->setWindowTitle(titleStr.trimmed());
@@ -290,7 +290,7 @@ CHtmlSysFrameQt::fRunGame()
             // Display a "game has ended" message. We use HTML, so Make sure
             // the parser is in markup mode.
             this->fParser->obey_markups(true);
-            QString endMsg(QString::fromAscii("<p><br><font face=tads-serif size=-1>(The game has ended.)</font></p>"));
+            QString endMsg(QString::fromLatin1("<p><br><font face=tads-serif size=-1>(The game has ended.)</font></p>"));
             this->display_output(endMsg.toUtf8().constData(), endMsg.length());
             this->flush_txtbuf(true, false);
         } else {
@@ -311,8 +311,8 @@ CHtmlSysFrameQt::fRunT2Game( const QString& fname )
 
     // T2 requires argc/argv style arguments.
     char argv0[] = "qtads";
-    char* argv1 = new char[QFile::encodeName(fname).size() + 1];
-    strcpy(argv1, QFile::encodeName(fname).constData());
+    char* argv1 = new char[qStrToFname(fname).size() + 1];
+    strcpy(argv1, qStrToFname(fname).constData());
     char* argv[2] = {argv0, argv1};
 
     // We always use .sav as the extension for T2 save files.
@@ -331,14 +331,14 @@ CHtmlSysFrameQt::fRunT3Game( const QString& fname )
     // vm_run_image_params doesn't copy the filename string but stores
     // the pointer directly.  We therefore need to hold a reference to
     // the data so that it won't go out of scope.
-    const QByteArray& fnameData = QFile::encodeName(fname);
+    const QByteArray& fnameData = qStrToFname(fname);
     vm_run_image_params params(this->fClientifc, this->fHostifc, fnameData.constData());
     this->fTads3 = true;
     vm_run_image(&params);
 }
 
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 #include <QFileOpenEvent>
 bool
 CHtmlSysFrameQt::event( QEvent* e )
@@ -475,27 +475,27 @@ CHtmlSysFrameQt::createFont( const CHtmlFontDesc* font_desc )
         // The face name field can contain multiple face names separated by
         // commas.  We split them into a list and try each one individualy.
         bool matchFound = false;
-        const QStringList& strList = QString(QString::fromAscii(newFontDesc.face))
-                                     .split(QChar::fromAscii(','), QString::SkipEmptyParts);
+        const QStringList& strList = QString(QString::fromLatin1(newFontDesc.face))
+                                     .split(QChar::fromLatin1(','), QString::SkipEmptyParts);
         for (int i = 0; i < strList.size() and not matchFound; ++i) {
             const QString& s = strList.at(i).simplified().toLower();
-            if (s == QString::fromAscii(HTMLFONT_TADS_SERIF).toLower()) {
+            if (s == QString::fromLatin1(HTMLFONT_TADS_SERIF).toLower()) {
                 fontName = this->fSettings->serifFont.family();
                 base_point_size = this->fSettings->serifFont.pointSize();
                 matchFound = true;
-            } else if (s == QString::fromAscii(HTMLFONT_TADS_SANS).toLower()) {
+            } else if (s == QString::fromLatin1(HTMLFONT_TADS_SANS).toLower()) {
                 fontName = this->fSettings->sansFont.family();
                 base_point_size = this->fSettings->sansFont.pointSize();
                 matchFound = true;
-            } else if (s == QString::fromAscii(HTMLFONT_TADS_SCRIPT).toLower()) {
+            } else if (s == QString::fromLatin1(HTMLFONT_TADS_SCRIPT).toLower()) {
                 fontName = this->fSettings->scriptFont.family();
                 base_point_size = this->fSettings->scriptFont.pointSize();
                 matchFound = true;
-            } else if (s == QString::fromAscii(HTMLFONT_TADS_TYPEWRITER).toLower()) {
+            } else if (s == QString::fromLatin1(HTMLFONT_TADS_TYPEWRITER).toLower()) {
                 fontName = this->fSettings->writerFont.family();
                 base_point_size = this->fSettings->writerFont.pointSize();
                 matchFound = true;
-            } else if (s == QString::fromAscii(HTMLFONT_TADS_INPUT).toLower()) {
+            } else if (s == QString::fromLatin1(HTMLFONT_TADS_INPUT).toLower()) {
                 fontName = this->fSettings->inputFont.family();
                 base_point_size = this->fSettings->inputFont.pointSize();
                 if (newFontDesc.face_set_explicitly) {
@@ -508,7 +508,7 @@ CHtmlSysFrameQt::createFont( const CHtmlFontDesc* font_desc )
                     newFont.color(HTML_COLOR_INPUT);
                 }
                 matchFound = true;
-            } else if (s == QString::fromAscii("qtads-grid")) {
+            } else if (s == QString::fromLatin1("qtads-grid")) {
                 // "qtads-grid" is an internal face name; it means we should
                 // return a font suitable for a text grid banner.
                 fontName = this->fSettings->fixedFont.family();
@@ -1017,7 +1017,7 @@ CHtmlSysFrameQt::pause_for_more()
 
 
 void
-CHtmlSysFrameQt::dbg_print( const char *msg )
+CHtmlSysFrameQt::dbg_print( const char* /*msg*/ )
 {
     //qDebug() << "HTML TADS Debug message:" << msg;
 }
@@ -1116,8 +1116,9 @@ CHtmlSysFrameQt::remove_banner_window( CHtmlSysWin* win )
 
 
 int
-CHtmlSysFrameQt::get_exe_resource( const textchar_t* resname, size_t resnamelen, textchar_t* fname_buf,
-                                   size_t fname_buf_len, unsigned long* seek_pos, unsigned long* siz )
+CHtmlSysFrameQt::get_exe_resource( const textchar_t* /*resname*/, size_t /*resnamelen*/,
+                                   textchar_t* /*fname_buf*/, size_t /*fname_buf_len*/,
+                                   unsigned long* /*seek_pos*/, unsigned long* /*siz*/ )
 {
     //qDebug() << Q_FUNC_INFO;
     //qDebug() << "resname:" << resname << "fname_buf:" << fname_buf << "seek_pos:" << seek_pos;
