@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -20,6 +20,7 @@
 #include <QDebug>
 
 #include "htmlsys.h"
+#include "config.h"
 
 
 /* Tads HTML layer class whose interface needs to be implemented by the
@@ -29,7 +30,7 @@
  * about this class.
  */
 class CHtmlSysSoundMidiQt: public CHtmlSysSoundMidi {
-#ifndef Q_WS_ANDROID
+#ifndef Q_OS_ANDROID
   private:
     struct SDL_RWops* fRWops;
     Mix_Music* fMusic;
@@ -53,10 +54,9 @@ class CHtmlSysSoundMidiQt: public CHtmlSysSoundMidi {
 #endif
 
   public:
-#ifndef Q_WS_ANDROID
+#ifndef Q_OS_ANDROID
     CHtmlSysSoundMidiQt( struct SDL_RWops* music );
-
-    virtual ~CHtmlSysSoundMidiQt();
+    ~CHtmlSysSoundMidiQt() override;
 
     // SDL_Mixer callback.
     static void callback();
@@ -65,19 +65,19 @@ class CHtmlSysSoundMidiQt: public CHtmlSysSoundMidi {
     //
     // CHtmlSysSoundMidi interface implementation.
     //
-    virtual int
+    int
     play_sound( CHtmlSysWin* win, void (*done_func)(void*, int repeat_count), void* done_func_ctx, int repeat,
-                const textchar_t* url, int vol, long fade_in, long fade_out, int crossfade );
+                const textchar_t* url, int vol, long fade_in, long fade_out, int crossfade ) override;
 
-    virtual void
-    add_crossfade( CHtmlSysWin* win, long ms )
+    void
+    add_crossfade( CHtmlSysWin*, long ) override
     { qDebug() << Q_FUNC_INFO; }
 
-    virtual void
-    cancel_sound( CHtmlSysWin* win, int sync, long fade_out_ms, int fade_in_bg );
+    void
+    cancel_sound( CHtmlSysWin* win, int sync, long fade_out_ms, int fade_in_bg ) override;
 
-    virtual int
-    maybe_suspend( CHtmlSysSound* )
+    int
+    maybe_suspend( CHtmlSysSound* ) override
     // MIDI is exclusive - we can only play one MIDI sound at a time.  However,
     // the current HTML TADS model only allows MIDI to play in one layer (the
     // background layer) anyway, so we should never find ourselves wanting to
@@ -90,8 +90,8 @@ class CHtmlSysSoundMidiQt: public CHtmlSysSoundMidi {
     // have a real implementation.
     { return false; }
 
-    virtual void
-    resume()
+    void
+    resume() override
     // We never suspend MIDI, so there's nothing to do.  (See the notes in
     // maybe_suspend() - if the model changes to require that MIDI suspension
     // be implemented, we would have to provide a real implementation here.)
