@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -19,6 +19,7 @@
 
 #include "qtadssound.h"
 #include "htmlsys.h"
+#include "config.h"
 
 
 /* Tads HTML layer class whose interface needs to be implemented by the
@@ -31,7 +32,7 @@ class CHtmlSysSoundMpegQt: public QTadsSound, public CHtmlSysSoundMpeg {
     Q_OBJECT
 
   public:
-#ifndef Q_WS_ANDROID
+#ifndef Q_OS_ANDROID
     CHtmlSysSoundMpegQt( QObject* parent, Mix_Chunk* chunk, SoundType type )
     : QTadsSound(parent, chunk, type)
     { }
@@ -40,24 +41,24 @@ class CHtmlSysSoundMpegQt: public QTadsSound, public CHtmlSysSoundMpeg {
     //
     // CHtmlSysSoundMpeg interface implementation.
     //
-    virtual int
+    int
     play_sound( CHtmlSysWin* win, void (*done_func)(void*, int repeat_count), void* done_func_ctx, int repeat,
-                const textchar_t* url, int vol, long fade_in, long fade_out, int crossfade );
+                const textchar_t* url, int vol, long fade_in, long fade_out, int crossfade ) override;
 
-    virtual void
-    add_crossfade( CHtmlSysWin* win, long ms );
+    void
+    add_crossfade( CHtmlSysWin* win, long ms ) override;
 
-    virtual void
-    cancel_sound( CHtmlSysWin* win, int sync, long fade_out_ms, int fade_in_bg );
+    void
+    cancel_sound( CHtmlSysWin* win, int sync, long fade_out_ms, int fade_in_bg ) override;
 
-    virtual int
-    maybe_suspend( CHtmlSysSound* )
+    int
+    maybe_suspend( CHtmlSysSound* ) override
     // We always return false since we have no limitation regarding the amount
     // of sounds we can play simultaneously.
     { return false; }
 
-    virtual void
-    resume();
+    void
+    resume() override;
 };
 
 
