@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -18,6 +18,7 @@
 #define SYSWINABOUTBOX_H
 
 #include "syswin.h"
+#include "config.h"
 
 
 /* We need special handling for the "About this game" box.
@@ -26,17 +27,17 @@ class CHtmlSysWinAboutBoxQt: public CHtmlSysWinQt {
     Q_OBJECT
 
   protected:
-    virtual void
-    keyPressEvent( QKeyEvent* e )
+    void
+    keyPressEvent( QKeyEvent* e ) override
     // It shouldn't be possible to do game input from the about box, so we
     // bypass the inherited input handling and revert to the default.
     { QScrollArea::keyPressEvent(e); }
 
-    virtual void
-    resizeEvent( QResizeEvent* e );
+    void
+    resizeEvent( QResizeEvent* e ) override;
 
-    virtual QSize
-    sizeHint() const;
+    QSize
+    sizeHint() const override;
 
   public:
     CHtmlSysWinAboutBoxQt( class CHtmlFormatter* formatter, QWidget* parent );
@@ -45,17 +46,17 @@ class CHtmlSysWinAboutBoxQt: public CHtmlSysWinQt {
     // width/height rather than our viewport's.  We need to do that because
     // the formatter needs to know our size before we become visible, and our
     // viewport only reports a valid size after show() is called.
-    virtual long
-    get_disp_width()
+    long
+    get_disp_width() override
     { return this->width(); }
 
-    virtual long
-    get_disp_height()
+    long
+    get_disp_height() override
     { return this->height(); }
 
-    virtual void
+    void
     set_banner_size( long width, HTML_BannerWin_Units_t width_units, int use_width,
-                     long height, HTML_BannerWin_Units_t height_units, int use_height );
+                     long height, HTML_BannerWin_Units_t height_units, int use_height ) override;
 };
 
 
