@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -35,7 +35,7 @@
 
 
 void
-QTadsFrame::resizeEvent( QResizeEvent* e )
+QTadsFrame::resizeEvent( QResizeEvent* )
 {
     qFrame->reformatBanners(true, true, false);
 }
@@ -62,16 +62,16 @@ CHtmlSysWinGroupQt::CHtmlSysWinGroupQt()
 
     // "Game" menu.
     QMenu* menu = menuBar->addMenu(tr("&Game"));
-    QAction* act = new QAction(tr("&Open") + QString::fromAscii("..."), this);
+    QAction* act = new QAction(tr("&Open") + QString::fromLatin1("..."), this);
 #if QT_VERSION >= 0x040600
-    act->setIcon(QIcon::fromTheme(QString::fromAscii("document-open")));
+    act->setIcon(QIcon::fromTheme(QString::fromLatin1("document-open")));
 #endif
     act->setShortcuts(QKeySequence::Open);
     menu->addAction(act);
     connect(act, SIGNAL(triggered()), this, SLOT(fOpenNewGame()));
     act = new QAction(tr("Open &Recent"), this);
 #if QT_VERSION >= 0x040600
-    act->setIcon(QIcon::fromTheme(QString::fromAscii("document-open-recent")));
+    act->setIcon(QIcon::fromTheme(QString::fromLatin1("document-open-recent")));
 #endif
     this->fRecentGamesMenu = new QMenu(this);
     act->setMenu(this->fRecentGamesMenu);
@@ -79,16 +79,16 @@ CHtmlSysWinGroupQt::CHtmlSysWinGroupQt()
     connect(this->fRecentGamesMenu, SIGNAL(triggered(QAction*)), this, SLOT(fRecentGameTriggered(QAction*)));
     this->fRestartCurrentGameAction = new QAction(tr("Re&start"), this);
 #if QT_VERSION >= 0x040600
-    this->fRestartCurrentGameAction->setIcon(QIcon::fromTheme(QString::fromAscii("view-refresh")));
+    this->fRestartCurrentGameAction->setIcon(QIcon::fromTheme(QString::fromLatin1("view-refresh")));
 #endif
-    this->fRestartCurrentGameAction->setShortcut(QKeySequence(QString::fromAscii("Ctrl+R")));
+    this->fRestartCurrentGameAction->setShortcut(QKeySequence(QString::fromLatin1("Ctrl+R")));
     menu->addAction(this->fRestartCurrentGameAction);
     this->fRestartCurrentGameAction->setEnabled(false);
     connect(this->fRestartCurrentGameAction, SIGNAL(triggered()), this, SLOT(fRestartCurrentGame()));
     this->fEndCurrentGameAction = new QAction(tr("Qui&t"), this);
     this->fEndCurrentGameAction->setMenuRole(QAction::NoRole);
 #if QT_VERSION >= 0x040600
-    this->fEndCurrentGameAction->setIcon(QIcon::fromTheme(QString::fromAscii("process-stop")));
+    this->fEndCurrentGameAction->setIcon(QIcon::fromTheme(QString::fromLatin1("process-stop")));
 #endif
     this->fEndCurrentGameAction->setShortcuts(QKeySequence::Close);
     menu->addAction(this->fEndCurrentGameAction);
@@ -102,7 +102,7 @@ CHtmlSysWinGroupQt::CHtmlSysWinGroupQt()
     connect(this->fAboutGameAction, SIGNAL(triggered()), this, SLOT(fShowAboutGame()));
     this->fGameInfoAction = new QAction(tr("View Metadata"), this);
 #if QT_VERSION >= 0x040600
-    this->fGameInfoAction->setIcon(QIcon::fromTheme(QString::fromAscii("document-properties")));
+    this->fGameInfoAction->setIcon(QIcon::fromTheme(QString::fromLatin1("document-properties")));
 #endif
     menu->addAction(this->fGameInfoAction);
     this->fGameInfoAction->setEnabled(false);
@@ -111,7 +111,7 @@ CHtmlSysWinGroupQt::CHtmlSysWinGroupQt()
     act = new QAction(tr("&Quit QTads"), this);
     act->setMenuRole(QAction::QuitRole);
 #if QT_VERSION >= 0x040600
-    act->setIcon(QIcon::fromTheme(QString::fromAscii("application-exit")));
+    act->setIcon(QIcon::fromTheme(QString::fromLatin1("application-exit")));
     act->setShortcuts(QKeySequence::Quit);
 #endif
     menu->addAction(act);
@@ -121,7 +121,7 @@ CHtmlSysWinGroupQt::CHtmlSysWinGroupQt()
     menu = menuBar->addMenu(tr("&Edit"));
     act = new QAction(tr("&Preferences..."), this);
 #if QT_VERSION >= 0x040600
-    act->setIcon(QIcon::fromTheme(QString::fromAscii("configure")));
+    act->setIcon(QIcon::fromTheme(QString::fromLatin1("configure")));
     act->setShortcuts(QKeySequence::Preferences);
 #endif
     menu->addAction(act);
@@ -131,7 +131,7 @@ CHtmlSysWinGroupQt::CHtmlSysWinGroupQt()
     menu = menuBar->addMenu(tr("&Help"));
     this->fAboutQtadsAction = new QAction(tr("A&bout QTads"), this);
 #if QT_VERSION >= 0x040600
-    this->fAboutQtadsAction->setIcon(QIcon::fromTheme(QString::fromAscii("help-about")));
+    this->fAboutQtadsAction->setIcon(QIcon::fromTheme(QString::fromLatin1("help-about")));
 #endif
     menu->addAction(this->fAboutQtadsAction);
     connect(this->fAboutQtadsAction, SIGNAL(triggered()), this, SLOT(fShowAboutQtads()));
@@ -181,7 +181,7 @@ CHtmlSysWinGroupQt::fAskQuitGameDialog()
     }
 
     QMessageBox* msgBox = new QMessageBox(QMessageBox::Question,
-                                          tr("Quit Current Game") + QString::fromAscii(" - ")
+                                          tr("Quit Current Game") + QString::fromLatin1(" - ")
                                           + qFrame->applicationName(),
                                           tr("If you didn't save the current game, all progress will"
                                              " be lost. Do you wish to quit the game?"),
@@ -190,8 +190,8 @@ CHtmlSysWinGroupQt::fAskQuitGameDialog()
     msgBox->setInformativeText(tr("This action will try to forcibly quit the game. Some games do not properly"
                                   " support this and can leave a stale process running in the background."
                                   " You should issue an in-game \"quit\" command in such cases."));
-#ifdef Q_WS_MAC
-    msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
+#ifdef Q_OS_MAC
+    msgBox->setIconPixmap(QPixmap(QString::fromLatin1(":/qtads_72x72.png")));
     // This presents the dialog as a sheet in OS X.
     msgBox->setWindowModality(Qt::WindowModal);
 #endif
@@ -211,7 +211,7 @@ CHtmlSysWinGroupQt::fAskRestartGameDialog()
     }
 
     QMessageBox* msgBox = new QMessageBox(QMessageBox::Question,
-                                          tr("Restart Current Game") + QString::fromAscii(" - ")
+                                          tr("Restart Current Game") + QString::fromLatin1(" - ")
                                           + qFrame->applicationName(),
                                           tr("If you didn't save the current game, all progress will be lost."
                                              " Do you wish to restart the game?"),
@@ -220,8 +220,8 @@ CHtmlSysWinGroupQt::fAskRestartGameDialog()
     msgBox->setInformativeText(tr("This action will try to forcibly quit and then restart the game. Some games"
                                   " do not properly support this and can leave a stale process running in the"
                                   " background. You should issue an in-game \"restart\" command in such cases."));
-#ifdef Q_WS_MAC
-    msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
+#ifdef Q_OS_MAC
+    msgBox->setIconPixmap(QPixmap(QString::fromLatin1(":/qtads_72x72.png")));
     // This presents the dialog as a sheet in OS X.
     msgBox->setWindowModality(Qt::WindowModal);
 #endif
@@ -244,7 +244,7 @@ CHtmlSysWinGroupQt::fCheckForUpdates()
     this->fNetManager = new QNetworkAccessManager(this);
     connect(this->fNetManager, SIGNAL(finished(QNetworkReply*)), SLOT(fReplyFinished(QNetworkReply*)));
 
-    this->fReply = this->fNetManager->get(QNetworkRequest(QUrl(QString::fromAscii("http://qtads.sourceforge.net/currentversion"))));
+    this->fReply = this->fNetManager->get(QNetworkRequest(QUrl(QString::fromLatin1("http://qtads.sourceforge.net/currentversion"))));
     connect(this->fReply, SIGNAL(error(QNetworkReply::NetworkError)),
             SLOT(fErrorOccurred(QNetworkReply::NetworkError)));
 }
@@ -260,21 +260,21 @@ CHtmlSysWinGroupQt::fReplyFinished( QNetworkReply* reply )
     }
 
     // Convert current version to hex.
-    QString str(QString::fromAscii(QTADS_VERSION));
+    QString str(QString::fromLatin1(QTADS_VERSION));
     // If this is a git snapshot, strip the " git".
-    if (str.endsWith(QString::fromAscii(" git"), Qt::CaseInsensitive)) {
+    if (str.endsWith(QString::fromLatin1(" git"), Qt::CaseInsensitive)) {
         str.chop(4);
     }
-    QStringList strList = str.split(QChar::fromAscii('.'));
+    QStringList strList = str.split(QChar::fromLatin1('.'));
     int curVersion = QT_VERSION_CHECK(strList.at(0).toInt(), strList.at(1).toInt(), strList.at(2).toInt());
 
     // Do the same with the retrieved version.
     str = QString::fromUtf8(reply->readLine(10));
     // Chop the newline at the end, if there is one.
-    if (str.endsWith(QChar::fromAscii('\n'))) {
+    if (str.endsWith(QChar::fromLatin1('\n'))) {
         str.chop(1);
     }
-    strList = str.split(QChar::fromAscii('.'));
+    strList = str.split(QChar::fromLatin1('.'));
     int newVersion = QT_VERSION_CHECK(strList.at(0).toInt(), strList.at(1).toInt(), strList.at(2).toInt());
 
     QMessageBox* msgBox = new QMessageBox(this);
@@ -289,14 +289,14 @@ CHtmlSysWinGroupQt::fReplyFinished( QNetworkReply* reply )
             text.append(QString::fromUtf8(reply->readLine(100)));
         }
         // Remove that last newline.
-        if (text.endsWith(QString::fromAscii("\n"))) {
+        if (text.endsWith(QString::fromLatin1("\n"))) {
             text.truncate(text.length() - 1);
         }
         if (text.length() > 2) {
             msgBox->setDetailedText(text);
         }
-#ifdef Q_WS_MAC
-        msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
+#ifdef Q_OS_MAC
+        msgBox->setIconPixmap(QPixmap(QString::fromLatin1(":/qtads_72x72.png")));
 #else
         msgBox->setIcon(QMessageBox::Question);
 #endif
@@ -306,11 +306,11 @@ CHtmlSysWinGroupQt::fReplyFinished( QNetworkReply* reply )
         msgBox->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
         msgBox->setDefaultButton(QMessageBox::Yes);
         if (msgBox->exec() == QMessageBox::Yes) {
-            QDesktopServices::openUrl(QUrl(QString::fromAscii("http://qtads.sourceforge.net/downloads.shtml")));
+            QDesktopServices::openUrl(QUrl(QString::fromLatin1("http://qtads.sourceforge.net/downloads.shtml")));
         }
     } else if (not this->fSilentIfNoUpdates) {
-#ifdef Q_WS_MAC
-        msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
+#ifdef Q_OS_MAC
+        msgBox->setIconPixmap(QPixmap(QString::fromLatin1(":/qtads_72x72.png")));
 #else
         msgBox->setIcon(QMessageBox::Information);
 #endif
@@ -325,7 +325,7 @@ CHtmlSysWinGroupQt::fReplyFinished( QNetworkReply* reply )
 }
 
 void
-CHtmlSysWinGroupQt::fErrorOccurred( QNetworkReply::NetworkError code )
+CHtmlSysWinGroupQt::fErrorOccurred( QNetworkReply::NetworkError )
 {
     QMessageBox* msg = new QMessageBox(QMessageBox::Critical, tr("Check for Updates - Error"),
                                        tr("It was not possible to retrieve update information. Please try again later,"
@@ -348,7 +348,7 @@ CHtmlSysWinGroupQt::fShowGameInfoDialog()
         this->fGameInfoDialog->raise();
         return;
     }
-    this->fGameInfoDialog = new GameInfoDialog(QFile::encodeName(qFrame->gameFile()), this);
+    this->fGameInfoDialog = new GameInfoDialog(qStrToFname(qFrame->gameFile()), this);
     this->fGameInfoDialog->setWindowTitle(tr("Game Information"));
     connect(this->fGameInfoDialog, SIGNAL(finished(int)), this, SLOT(fHideGameInfoDialog()));
     this->fGameInfoDialog->show();
@@ -377,7 +377,7 @@ CHtmlSysWinGroupQt::fShowConfDialog()
     this->fConfDialog = new ConfDialog(this);
     this->fConfDialog->setWindowTitle(tr("QTads Preferences"));
     connect(this->fConfDialog, SIGNAL(finished(int)), this, SLOT(fHideConfDialog()));
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     // There's a bug in Qt for OS X that results in a visual glitch with
     // QFontComboBox widgets inside QFormLayouts.  Making the dialog 4 pixels
     // higher fixes it.
@@ -449,7 +449,7 @@ CHtmlSysWinGroupQt::fShowAboutQtads()
 
     this->fAboutQtadsDialog = new AboutQtadsDialog(this);
     connect(this->fAboutQtadsDialog, SIGNAL(finished(int)), this, SLOT(fHideAboutQtads()));
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     // Similar bug to the config dialog one.  Again, 4 pixels higher fixes it.
     this->fAboutQtadsDialog->layout()->activate();
     this->fAboutQtadsDialog->setMinimumHeight(this->fAboutQtadsDialog->minimumHeight() + 4);
@@ -474,7 +474,7 @@ CHtmlSysWinGroupQt::fOpenNewGame()
     const QString& fname = QFileDialog::getOpenFileName(0, tr("Choose the TADS game you wish to run"),
                                                         qFrame->settings()->lastFileOpenDir,
                                                         tr("TADS Games")
-                                                        + QString::fromAscii(" (*.gam *.Gam *.GAM *.t3 *.T3)"));
+                                                        + QString::fromLatin1(" (*.gam *.Gam *.GAM *.t3 *.T3)"));
     if (not fname.isEmpty()) {
         qFrame->settings()->lastFileOpenDir = QFileInfo(fname).absolutePath();
         qFrame->setNextGame(fname);
@@ -522,7 +522,7 @@ void
 CHtmlSysWinGroupQt::fNotifyGameStarting()
 {
     this->fHideGameInfoDialog();
-    this->fGameInfoAction->setEnabled(GameInfoDialog::gameHasMetaInfo(QFile::encodeName(qFrame->gameFile())));
+    this->fGameInfoAction->setEnabled(GameInfoDialog::gameHasMetaInfo(qStrToFname(qFrame->gameFile())));
     this->fRestartCurrentGameAction->setEnabled(true);
     this->fEndCurrentGameAction->setEnabled(true);
 }
@@ -543,8 +543,8 @@ CHtmlSysWinGroupQt::closeEvent( QCloseEvent* e )
     msgBox->setInformativeText(tr("This action will try to forcibly quit the game. Some games do not properly"
                                   " support this and can leave a stale process running in the background."
                                   " You should issue an in-game \"quit\" command first in such cases."));
-#ifdef Q_WS_MAC
-    msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
+#ifdef Q_OS_MAC
+    msgBox->setIconPixmap(QPixmap(QString::fromLatin1(":/qtads_72x72.png")));
     // This presents the dialog as a sheet in OS X.
     msgBox->setWindowModality(Qt::WindowModal);
 #endif
@@ -643,11 +643,11 @@ CHtmlSysWinGroupQt::updateRecentGames()
     // Populate it.
     const QStringList& list = qFrame->settings()->recentGamesList;
     for (int i = 0; i < list.size(); ++i) {
-        QString gameName = GameInfoDialog::getMetaInfo(QFile::encodeName(list.at(i))).plainGameName;
+        QString gameName = GameInfoDialog::getMetaInfo(qStrToFname(list.at(i))).plainGameName;
         if (gameName.isEmpty()) {
             gameName = QFileInfo(list.at(i)).fileName();
         }
-        gameName = gameName.replace(QString::fromAscii("&"), QString::fromAscii("&&"));
+        gameName = gameName.replace(QString::fromLatin1("&"), QString::fromLatin1("&&"));
         QAction* act = this->fRecentGamesMenu->addAction(gameName);
         // Elide the text in case it's too long.
         act->setText(QFontMetrics(act->font()).elidedText(gameName, Qt::ElideRight, 300));
@@ -663,7 +663,7 @@ CHtmlSysWinGroupQt::checkForUpdates()
 }
 
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 bool
 CHtmlSysWinGroupQt::handleFileOpenEvent( class QFileOpenEvent* e )
 {
@@ -690,7 +690,7 @@ CHtmlSysWinGroupQt::get_default_win_charset() const
 
 size_t
 CHtmlSysWinGroupQt::xlat_html4_entity( textchar_t* result, size_t result_size, unsigned int charval,
-                                       oshtml_charset_id_t* charset, int* changed_charset )
+                                       oshtml_charset_id_t*, int* changed_charset )
 {
     //qDebug() << Q_FUNC_INFO << "called";
     Q_ASSERT(result != 0);
