@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Nikos Chantziaras.
+/* Copyright (C) 2013 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free software; you
  * can redistribute it and/or modify it under the terms of the GNU General
@@ -141,7 +141,7 @@ CHtmlSysWinInputQt::processCommand( const textchar_t* cmd, size_t len, int appen
         // If the HREF string is empty, use a single space so that we know that
         // an HREF event actually occured.
         if (cmd[0] == '\0') {
-            this->fHrefEvent = QChar::fromAscii(' ');
+            this->fHrefEvent = QChar::fromLatin1(' ');
         } else {
             this->fHrefEvent = QString::fromUtf8(cmd);
         }
